//
//    Copyright 2004 - 2007 Thomas C. McDermott, N5EG
//    This file is part of VNAR - the Vector Network Analyzer program.
//
//    VNAR is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    VNAR is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with VNAR, if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#pragma once

//	Routines to encapsulate directional coupler compensation


#include "stdafx.h"
#include "DisplayRoutines.h"
#using <mscorlib.dll>
#using <System.dll>
using namespace System;
using namespace System::ComponentModel;
using namespace System::IO;
using namespace System::Windows::Forms;

__gc public class CalDataSet;			//so DirectionalCoupler class can forward-reference CalDataSet

__gc public struct PerErrModelCoeff {	// set of coefficients for 6-point error model
	int freq;							// frequency where set applies
	double offset;						// offset angle of the correction in degrees
	double mag;							// peak magnitude of the correction
};

__gc public class DirectionalCoupler
{

public:

	//	double calCableSurgeRealOhms, calCableSurgeImagOhms;	// real and imaginary parts of cal cable surge impedance

	int DirMag __gc[];					// Directivity cal raw data
	int DirIphs __gc[];					// Directivity cal raw data
	int DirQphs __gc[];					// Directivity cal raw data

	PerErrModelCoeff* PhRipCoeff __gc[];	// Array of phase ripple coefficients
	PerErrModelCoeff* MagRipCoeff __gc[];	// Array of mag ripple coefficients

	double phaseError __gc[];			// Array of open-short angle error for LF phase computation
	double magError __gc[];				// Array of open-short magnitude for magnitude compensation
	double openAngle __gc[];			// Array of open angle for LF phase computation
	double shortAngle __gc[];			// Array of short angle for LF phase computation

	bool DirCalibrated;					///< true if Directivity Calibration has been run or values loaded from cal file
	bool RippleCalibrated;				///< true if Mag & Phase ripple calibration has been run or values loaded from cal file

	double PhaseDCOffset;				///< Value of phase error offset (sum of all  phaseError/numPoints)
	double MagDCOffset;					///< Value of mag error offset (sum of magError/numPoints)


DirectionalCoupler(void);	// constructor
	// store coupler directivity calibration raw readings
void DirectivityCal(int __gc [,]);
	// Compensate S11 reading for Coupler Directivity
void CompensateDirectivity(CalDataSet* cal, double& rmagnitudeLin, double& rphase, int Freq); 
	// Generate table of unit-length ripple compensation vectors
bool GenerateShortOpenComp(CalDataSet* cal, int shortedrefl __gc [,], int openrefl __gc[,]);
double PhaseRippleCompensate(double phase, int frequency);
	// Compensate coupler magnitude ripple error   NEW 09-23-2007
double MagRippleCorrection(double phase, int frequency);
	// Update parameter estimates by descending along the gradient   NEW 09-23-2007
void DirectionalCoupler::Descend(double partialDerivative __nogc[], double weight __nogc[], PerErrModelCoeff* __gc[]);
	// Compute error predicted by point-wise periodic error model
double DirectionalCoupler::PeriodicModelCorrection(int frequency, double phase, PerErrModelCoeff* __gc[], double DCOffset);
	// Compute SumSqErr for model compared to error signature
double DirectionalCoupler::SumSquareError(int startI, int stopI, double angle __gc[],
		double errSignature __gc[], PerErrModelCoeff* __gc[], double DCOffset);
	// Compute gradient of the model parameter set vs. the SumSqError of the fit
void DirectionalCoupler::Gradient(double partialDerivative __nogc[], int startI, int stopI,
		double angle __gc[], double errSignature __gc[], PerErrModelCoeff*  modelCoeff __gc[], double DCOffset);

};

